﻿#!/usr/bin/env python
# models.py

# adapted from tornado-sqlalchemy-template: https://github.com/haldun/tornado-sqlalchemy-template

from sqlalchemy import *
from sqlalchemy.orm import scoped_session, sessionmaker, relation, relationship
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.dialects.sqlite import *

Base = declarative_base()

def init_db(engine):
    Base.metadata.create_all(bind=engine)
    
CategoryTable = Table(u'Categories', Base.metadata,

    #column definitions
    Column(u'CategoryID', INTEGER(), Sequence('seq_categories_categoryid'), primary_key=True, nullable=False),
    Column(u'CategoryName', NVARCHAR(length=15), nullable=False),
    Column(u'Description', UnicodeText(length=1073741823)),
    Column(u'Picture', BLOB()),

    #foreign keys

    #unique constraints
    UniqueConstraint('CategoryName',name='CategoryName'),

    # __LLBLGENPRO_USER_CODE_REGION_START additional_table_information_categories 
    # __LLBLGENPRO_USER_CODE_REGION_END 
)

CustomerTable = Table(u'Customers', Base.metadata,

    #column definitions
    Column(u'CustomerID', NCHAR(length=5), primary_key=True, nullable=False),
    Column(u'CompanyName', NVARCHAR(length=40), nullable=False),
    Column(u'ContactName', NVARCHAR(length=30)),
    Column(u'ContactTitle', NVARCHAR(length=30)),
    Column(u'Address', NVARCHAR(length=60)),
    Column(u'City', NVARCHAR(length=15)),
    Column(u'Region', NVARCHAR(length=15)),
    Column(u'PostalCode', NVARCHAR(length=10)),
    Column(u'Country', NVARCHAR(length=15)),
    Column(u'Phone', NVARCHAR(length=24)),
    Column(u'Fax', NVARCHAR(length=24)),

    #foreign keys

    #unique constraints
    UniqueConstraint('CompanyName',name='CompanyName'),

    # __LLBLGENPRO_USER_CODE_REGION_START additional_table_information_customers 
    # __LLBLGENPRO_USER_CODE_REGION_END 
)

CustomerCustomerDemoTable = Table(u'CustomerCustomerDemo', Base.metadata,

    #column definitions
    Column(u'CustomerID', NCHAR(length=5), primary_key=True, nullable=False),
    Column(u'CustomerTypeID', NCHAR(length=10), primary_key=True, nullable=False),

    #foreign keys
    ForeignKeyConstraint(['CustomerTypeID'],['CustomerDemographics.CustomerTypeID'],name='FK_CustomerCustomerDemo'),
    ForeignKeyConstraint(['CustomerID'],['Customers.CustomerID'],name='FK_CustomerCustomerDemo_Customers'),

    #unique constraints

    # __LLBLGENPRO_USER_CODE_REGION_START additional_table_information_customercustomerdemo 
    # __LLBLGENPRO_USER_CODE_REGION_END 
)

CustomerDemographicTable = Table(u'CustomerDemographics', Base.metadata,

    #column definitions
    Column(u'CustomerTypeID', NCHAR(length=10), primary_key=True, nullable=False),
    Column(u'CustomerDesc', UnicodeText(length=1073741823)),

    #foreign keys

    #unique constraints

    # __LLBLGENPRO_USER_CODE_REGION_START additional_table_information_customerdemographics 
    # __LLBLGENPRO_USER_CODE_REGION_END 
)

EmployeeTable = Table(u'Employees', Base.metadata,

    #column definitions
    Column(u'EmployeeID', INTEGER(), Sequence('seq_employees_employeeid'), primary_key=True, nullable=False),
    Column(u'LastName', NVARCHAR(length=20), nullable=False),
    Column(u'FirstName', NVARCHAR(length=10), nullable=False),
    Column(u'Title', NVARCHAR(length=30)),
    Column(u'TitleOfCourtesy', NVARCHAR(length=25)),
    Column(u'BirthDate', DATETIME()),
    Column(u'HireDate', DATETIME()),
    Column(u'Address', NVARCHAR(length=60)),
    Column(u'City', NVARCHAR(length=15)),
    Column(u'Region', NVARCHAR(length=15)),
    Column(u'PostalCode', NVARCHAR(length=10)),
    Column(u'Country', NVARCHAR(length=15)),
    Column(u'HomePhone', NVARCHAR(length=24)),
    Column(u'Extension', NVARCHAR(length=4)),
    Column(u'Photo', BLOB()),
    Column(u'Notes', UnicodeText(length=1073741823)),
    Column(u'ReportsTo', INTEGER()),
    Column(u'PhotoPath', NVARCHAR(length=255)),

    #foreign keys
    ForeignKeyConstraint(['ReportsTo'],['Employees.EmployeeID'],name='FK_Employees_Employees'),

    #unique constraints

    # __LLBLGENPRO_USER_CODE_REGION_START additional_table_information_employees 
    # __LLBLGENPRO_USER_CODE_REGION_END 
)

EmployeeTerritoryTable = Table(u'EmployeeTerritories', Base.metadata,

    #column definitions
    Column(u'EmployeeID', INTEGER(), primary_key=True, nullable=False),
    Column(u'TerritoryID', NVARCHAR(length=20), primary_key=True, nullable=False),

    #foreign keys
    ForeignKeyConstraint(['EmployeeID'],['Employees.EmployeeID'],name='FK_EmployeeTerritories_Employees'),
    ForeignKeyConstraint(['TerritoryID'],['Territories.TerritoryID'],name='FK_EmployeeTerritories_Territories'),

    #unique constraints

    # __LLBLGENPRO_USER_CODE_REGION_START additional_table_information_employeeterritories 
    # __LLBLGENPRO_USER_CODE_REGION_END 
)

OrderTable = Table(u'Orders', Base.metadata,

    #column definitions
    Column(u'OrderID', INTEGER(), Sequence('seq_orders_orderid'), primary_key=True, nullable=False),
    Column(u'CustomerID', NCHAR(length=5)),
    Column(u'EmployeeID', INTEGER()),
    Column(u'OrderDate', DATETIME()),
    Column(u'RequiredDate', DATETIME()),
    Column(u'ShippedDate', DATETIME()),
    Column(u'ShipVia', INTEGER()),
    Column(u'Freight', DECIMAL(precision=19, scale=4)),
    Column(u'ShipName', NVARCHAR(length=40)),
    Column(u'ShipAddress', NVARCHAR(length=60)),
    Column(u'ShipCity', NVARCHAR(length=15)),
    Column(u'ShipRegion', NVARCHAR(length=15)),
    Column(u'ShipPostalCode', NVARCHAR(length=10)),
    Column(u'ShipCountry', NVARCHAR(length=15)),

    #foreign keys
    ForeignKeyConstraint(['CustomerID'],['Customers.CustomerID'],name='FK_Orders_Customers'),
    ForeignKeyConstraint(['EmployeeID'],['Employees.EmployeeID'],name='FK_Orders_Employees'),
    ForeignKeyConstraint(['ShipVia'],['Shippers.ShipperID'],name='FK_Orders_Shippers'),

    #unique constraints

    # __LLBLGENPRO_USER_CODE_REGION_START additional_table_information_orders 
    # __LLBLGENPRO_USER_CODE_REGION_END 
)

OrderDetailTable = Table(u'OrderDetails', Base.metadata,

    #column definitions
    Column(u'OrderID', INTEGER(), primary_key=True, nullable=False),
    Column(u'ProductID', INTEGER(), primary_key=True, nullable=False),
    Column(u'UnitPrice', DECIMAL(precision=19, scale=4), nullable=False),
    Column(u'Quantity', SMALLINT(), nullable=False),
    Column(u'Discount', REAL(precision=24), nullable=False),

    #foreign keys
    ForeignKeyConstraint(['OrderID'],['Orders.OrderID'],name='FK_Order_Details_Orders'),
    ForeignKeyConstraint(['ProductID'],['Products.ProductID'],name='FK_Order_Details_Products'),

    #unique constraints

    # __LLBLGENPRO_USER_CODE_REGION_START additional_table_information_orderdetails 
    # __LLBLGENPRO_USER_CODE_REGION_END 
)

ProductTable = Table(u'Products', Base.metadata,

    #column definitions
    Column(u'ProductID', INTEGER(), Sequence('seq_products_productid'), primary_key=True, nullable=False),
    Column(u'ProductName', NVARCHAR(length=40), nullable=False),
    Column(u'SupplierID', INTEGER()),
    Column(u'CategoryID', INTEGER()),
    Column(u'QuantityPerUnit', NVARCHAR(length=20)),
    Column(u'UnitPrice', DECIMAL(precision=19, scale=4)),
    Column(u'UnitsInStock', SMALLINT()),
    Column(u'UnitsOnOrder', SMALLINT()),
    Column(u'ReorderLevel', SMALLINT()),
    Column(u'Discontinued', BOOLEAN(), nullable=False),

    #foreign keys
    ForeignKeyConstraint(['CategoryID'],['Categories.CategoryID'],name='FK_Products_Categories'),
    ForeignKeyConstraint(['SupplierID'],['Suppliers.SupplierID'],name='FK_Products_Suppliers'),

    #unique constraints
    UniqueConstraint('ProductName',name='ProductName'),

    # __LLBLGENPRO_USER_CODE_REGION_START additional_table_information_products 
    # __LLBLGENPRO_USER_CODE_REGION_END 
)

RegionTable = Table(u'Region', Base.metadata,

    #column definitions
    Column(u'RegionID', INTEGER(), primary_key=True, nullable=False),
    Column(u'RegionDescription', NCHAR(length=50), nullable=False),

    #foreign keys

    #unique constraints
    UniqueConstraint('RegionDescription',name='IX_Region'),

    # __LLBLGENPRO_USER_CODE_REGION_START additional_table_information_region 
    # __LLBLGENPRO_USER_CODE_REGION_END 
)

ShipperTable = Table(u'Shippers', Base.metadata,

    #column definitions
    Column(u'ShipperID', INTEGER(), Sequence('seq_shippers_shipperid'), primary_key=True, nullable=False),
    Column(u'CompanyName', NVARCHAR(length=40), nullable=False),
    Column(u'Phone', NVARCHAR(length=24)),

    #foreign keys

    #unique constraints

    # __LLBLGENPRO_USER_CODE_REGION_START additional_table_information_shippers 
    # __LLBLGENPRO_USER_CODE_REGION_END 
)

SupplierTable = Table(u'Suppliers', Base.metadata,

    #column definitions
    Column(u'SupplierID', INTEGER(), Sequence('seq_suppliers_supplierid'), primary_key=True, nullable=False),
    Column(u'CompanyName', NVARCHAR(length=40), nullable=False),
    Column(u'ContactName', NVARCHAR(length=30)),
    Column(u'ContactTitle', NVARCHAR(length=30)),
    Column(u'Address', NVARCHAR(length=60)),
    Column(u'City', NVARCHAR(length=15)),
    Column(u'Region', NVARCHAR(length=15)),
    Column(u'PostalCode', NVARCHAR(length=10)),
    Column(u'Country', NVARCHAR(length=15)),
    Column(u'Phone', NVARCHAR(length=24)),
    Column(u'Fax', NVARCHAR(length=24)),
    Column(u'HomePage', UnicodeText(length=1073741823)),

    #foreign keys

    #unique constraints
    UniqueConstraint('CompanyName',name='SupplierName'),

    # __LLBLGENPRO_USER_CODE_REGION_START additional_table_information_suppliers 
    # __LLBLGENPRO_USER_CODE_REGION_END 
)

TerritoryTable = Table(u'Territories', Base.metadata,

    #column definitions
    Column(u'TerritoryID', NVARCHAR(length=20), primary_key=True, nullable=False),
    Column(u'TerritoryDescription', NCHAR(length=50), nullable=False),
    Column(u'RegionID', INTEGER(), nullable=False),

    #foreign keys
    ForeignKeyConstraint(['RegionID'],['Region.RegionID'],name='FK_Territories_Region'),

    #unique constraints

    # __LLBLGENPRO_USER_CODE_REGION_START additional_table_information_territories 
    # __LLBLGENPRO_USER_CODE_REGION_END 
)

class Category(Base):
    __table__ = CategoryTable

    #relation definitions: one to many
    #see backref mapping on Product class --> Products = relationship('Product', backref='Category')

    # __LLBLGENPRO_USER_CODE_REGION_START additional_model_information_category 
    # __LLBLGENPRO_USER_CODE_REGION_END 

class Customer(Base):
    __table__ = CustomerTable

    #relation definitions: one to many
    #see backref mapping on CustomerCustomerDemo class --> CustomerCustomerDemos = relationship('CustomerCustomerDemo', backref='Customer')
    #see backref mapping on Order class --> Orders = relationship('Order', backref='Customer')

    # __LLBLGENPRO_USER_CODE_REGION_START additional_model_information_customer 
    # __LLBLGENPRO_USER_CODE_REGION_END 

class CustomerCustomerDemo(Base):
    __table__ = CustomerCustomerDemoTable

    #relation definitions: many to one with backref (also takes care of one to many)
    Customer = relationship('Customer', backref='CustomerCustomerDemos')
    CustomerDemographic = relationship('CustomerDemographic', backref='CustomerCustomerDemos')

    # __LLBLGENPRO_USER_CODE_REGION_START additional_model_information_customercustomerdemo 
    # __LLBLGENPRO_USER_CODE_REGION_END 

class CustomerDemographic(Base):
    __table__ = CustomerDemographicTable

    #relation definitions: one to many
    #see backref mapping on CustomerCustomerDemo class --> CustomerCustomerDemos = relationship('CustomerCustomerDemo', backref='CustomerDemographic')

    # __LLBLGENPRO_USER_CODE_REGION_START additional_model_information_customerdemographic 
    # __LLBLGENPRO_USER_CODE_REGION_END 

class Employee(Base):
    __table__ = EmployeeTable

    #relation definitions: one to many
    #see backref mapping on Employee class --> Employees = relationship('Employee', backref='Manager')
    #see backref mapping on EmployeeTerritory class --> EmployeeTerritories = relationship('EmployeeTerritory', backref='Employee')
    #see backref mapping on Order class --> Orders = relationship('Order', backref='Employee')

    #relation definitions: many to many with backref
    Territories = relationship('Territory', secondary=EmployeeTerritoryTable, backref='Employees')

    #relation definitions: many to one with backref (also takes care of one to many)
    Manager = relationship('Employee', primaryjoin=(EmployeeTable.c.EmployeeID==EmployeeTable.c.ReportsTo), remote_side='Employee.EmployeeID', backref='Employees')

    # __LLBLGENPRO_USER_CODE_REGION_START additional_model_information_employee 
    # __LLBLGENPRO_USER_CODE_REGION_END 

class EmployeeTerritory(Base):
    __table__ = EmployeeTerritoryTable

    #relation definitions: many to one with backref (also takes care of one to many)
    Employee = relationship('Employee', backref='EmployeeTerritories')
    Territory = relationship('Territory', backref='EmployeeTerritories')

    # __LLBLGENPRO_USER_CODE_REGION_START additional_model_information_employeeterritory 
    # __LLBLGENPRO_USER_CODE_REGION_END 

class Order(Base):
    __table__ = OrderTable

    #relation definitions: one to many
    #see backref mapping on OrderDetail class --> OrderDetails = relationship('OrderDetail', backref='Order')

    #relation definitions: many to many with backref
    ProductCollectionViaOrderDetail = relationship('OrderDetail', backref='OrderCollectionViaOrderDetail')

    #relation definitions: many to one with backref (also takes care of one to many)
    Customer = relationship('Customer', backref='Orders')
    Employee = relationship('Employee', backref='Orders')
    Shipper = relationship('Shipper', backref='Orders')

    # __LLBLGENPRO_USER_CODE_REGION_START additional_model_information_order 
    # __LLBLGENPRO_USER_CODE_REGION_END 

class OrderDetail(Base):
    __table__ = OrderDetailTable

    #relation definitions: many to one with backref (also takes care of one to many)
    Order = relationship('Order', backref='OrderDetails')
    Product = relationship('Product', backref='OrderDetails')

    # __LLBLGENPRO_USER_CODE_REGION_START additional_model_information_orderdetail 
    # __LLBLGENPRO_USER_CODE_REGION_END 

class Product(Base):
    __table__ = ProductTable

    #relation definitions: one to many
    #see backref mapping on OrderDetail class --> OrderDetails = relationship('OrderDetail', backref='Product')

    #relation definitions: many to many with backref
    #see backref mapping on Order class --> OrderCollectionViaOrderDetail = relationship('OrderDetail', backref='ProductCollectionViaOrderDetail')

    #relation definitions: many to one with backref (also takes care of one to many)
    Category = relationship('Category', backref='Products')
    Supplier = relationship('Supplier', backref='Products')

    # __LLBLGENPRO_USER_CODE_REGION_START additional_model_information_product 
    # __LLBLGENPRO_USER_CODE_REGION_END 

class Region(Base):
    __table__ = RegionTable

    #relation definitions: one to many
    #see backref mapping on Territory class --> Territories = relationship('Territory', backref='Region')

    # __LLBLGENPRO_USER_CODE_REGION_START additional_model_information_region 
    # __LLBLGENPRO_USER_CODE_REGION_END 

class Shipper(Base):
    __table__ = ShipperTable

    #relation definitions: one to many
    #see backref mapping on Order class --> Orders = relationship('Order', backref='Shipper')

    # __LLBLGENPRO_USER_CODE_REGION_START additional_model_information_shipper 
    # __LLBLGENPRO_USER_CODE_REGION_END 

class Supplier(Base):
    __table__ = SupplierTable

    #relation definitions: one to many
    #see backref mapping on Product class --> Products = relationship('Product', backref='Supplier')

    # __LLBLGENPRO_USER_CODE_REGION_START additional_model_information_supplier 
    # __LLBLGENPRO_USER_CODE_REGION_END 

class Territory(Base):
    __table__ = TerritoryTable

    #relation definitions: one to many
    #see backref mapping on EmployeeTerritory class --> EmployeeTerritories = relationship('EmployeeTerritory', backref='Territory')

    #relation definitions: many to many with backref
    #see backref mapping on Employee class --> Employees = relationship('Employee', secondary=EmployeeTerritoryTable, backref='Territories')

    #relation definitions: many to one with backref (also takes care of one to many)
    Region = relationship('Region', backref='Territories')

    # __LLBLGENPRO_USER_CODE_REGION_START additional_model_information_territory 
    # __LLBLGENPRO_USER_CODE_REGION_END 


def main():
    engine = create_engine('sqlite://')
    init_db(engine)
    db = scoped_session(sessionmaker(bind=engine))
    #try a count on each table to make sure it's setup properly
    counts = [
        str(db.query(Category).count()) + " Category records",
        str(db.query(Customer).count()) + " Customer records",
        str(db.query(CustomerCustomerDemo).count()) + " CustomerCustomerDemo records",
        str(db.query(CustomerDemographic).count()) + " CustomerDemographic records",
        str(db.query(Employee).count()) + " Employee records",
        str(db.query(EmployeeTerritory).count()) + " EmployeeTerritory records",
        str(db.query(Order).count()) + " Order records",
        str(db.query(OrderDetail).count()) + " OrderDetail records",
        str(db.query(Product).count()) + " Product records",
        str(db.query(Region).count()) + " Region records",
        str(db.query(Shipper).count()) + " Shipper records",
        str(db.query(Supplier).count()) + " Supplier records",
        str(db.query(Territory).count()) + " Territory records",
    ]
    for c in counts:
        print(c)

if __name__ == '__main__':
  main()



